﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class SectionItemService : ISectionItemService
    {
        private readonly IMapper _mapper;
        private readonly Tsi1Context _context;
        private readonly IFileService _fileService;

        public SectionItemService(IMapper mapper, Tsi1Context context, IFileService fileService)
        {
            _mapper = mapper;
            _context = context;
            _fileService = fileService;
        }

        public async Task<ServiceResult<bool>> Create(SectionItemCreateDto newSectionItem)
        {
            var sectionItem = _mapper.Map<SectionItem>(newSectionItem);

            var result = await this.SectionItemValidations(sectionItem);

            if (result.HasError)
            {
                return result;
            }

            _context.SectionItems.Add(sectionItem);
            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> Delete(int sectionItemId)
        {
            var result = new ServiceResult<bool>();

            var sectionItem = await _context.SectionItems.FirstOrDefaultAsync(x => x.Id == sectionItemId);
            if (sectionItem == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.SectionItemDoesNotExist, sectionItemId));
                return result;
            }

            _context.SectionItems.Remove(sectionItem);
            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> OrderSectionItems(List<OrderDto> orderDtos)
        {
            var result = new ServiceResult<bool>();

            var sectionItemIds = orderDtos.Select(x => x.Id);
            var orders = orderDtos.Select(x => x.Order).Distinct();

            if (orders.Count() != orderDtos.Count())
            {
                result.HasError = true;
                result.AddMessage(ErrorMessages.DuplicateSectionItemOrder);
                return result;
            }

            var sectionItems = await _context.SectionItems
                .Where(x => sectionItemIds.Contains(x.Id))
                .ToListAsync();

            foreach (var orderDto in orderDtos)
            {
                var sectionItem = sectionItems.FirstOrDefault(x => x.Id == orderDto.Id);
                if (sectionItem == null)
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.SectionItemDoesNotExist, orderDto.Id));
                }

                sectionItem.Order = orderDto.Order;
            }

            if (!result.HasError)
            {
                await _context.SaveChangesAsync(); 
            }

            return result;
        }

        private async Task<ServiceResult<bool>> SectionItemValidations(SectionItem sectionItem)
        {
            var result = new ServiceResult<bool>();

            var sectionItemType = await _context.SectionItemTypes
                .FirstOrDefaultAsync(x => x.Id == sectionItem.SectionItemTypeId);

            if (sectionItemType == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.InvalidSectionItemType, sectionItem.SectionItemTypeId));
                return result;
            }

            if (sectionItemType.Name == SectionItemTypes.Forum)
            {
                if (sectionItem.Forum == null)
                {
                    result.HasError = true;
                    result.AddMessage(ErrorMessages.ForumIsNull);
                }

                if (sectionItem.File != null)
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.InvalidSectionItemData, sectionItemType.Name, SectionItemTypes.File));
                }
            }

            if (sectionItemType.Name == SectionItemTypes.File)
            {
                if (sectionItem.File == null)
                {
                    result.HasError = true;
                    result.AddMessage(ErrorMessages.FileIsNull);
                }

                if (sectionItem.Forum != null)
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.InvalidSectionItemData, sectionItemType.Name, SectionItemTypes.Forum));
                }

                if (!_fileService.ExistFile(sectionItem.File.Path))
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.FileDoesNotExist, sectionItem.File.Path));
                }
            }

            return result;
        }
    }
}
